@@ -16,7 +16,8 @@ const	Filters = {
 			ACTION_SCORE: 6,
 			ACTION_LABEL: 7,
 			ACTION_PLUGIN: 9,
-			PARAM_ACTIONS: [4, 6, 7, 9],
+			ACTION_REMOVE_TAG: 10,
+			PARAM_ACTIONS: [4, 6, 7, 9, 10],
 			filter_info: {},
 			test: function() {
 				const test_dialog = new fox.SingleUseDialog({
@@ -397,6 +398,8 @@ const	Filters = {
 					xhr.post("backend.php", this.attr('value'), () => {
 						dialog.hide();
 
+						Notify.close();
+
 						const tree = dijit.byId("filterTree");
 						if (tree) tree.reload();
 					});
